<?php

/*
 * Anewt, Almost No Effort Web Toolkit, form module
 *
 * This code is copyrighted and distributed under the terms of the GNU LGPL.
 * See the README file for more information.
 */


mkenum('ANEWT_FORM_CONTROL_TEXT_TYPE_TEXT', 'ANEWT_FORM_CONTROL_TEXT_TYPE_INT', 'ANEWT_FORM_CONTROL_TEXT_TYPE_FLOAT');

/**
 * \private
 *
 * Base form control for text input.
 *
 * This control supports both single line and multiline input and renderers
 * a HTML <code>input</code> or <code>textarea</code> widget.
 *
 * Do not instantiate this class, use one of its descendants instead.
 */
abstract class AnewtFormControlTextBase extends AnewtFormControl
{
	/**
	 * Create a new base text control.
	 *
	 * This constructor must be called from subclass constructors, e.g.
	 * AnewtFormControlText, AnewtFormControlTextMultiline, and
	 * AnewtFormControlPassword.
	 *
	 * \param $name
	 *   The name of this control.
	 */
	function __construct($name)
	{
		parent::__construct($name);
		$this->_seed(array(
			'multiline'  => false,
			'type'       => ANEWT_FORM_CONTROL_TEXT_TYPE_TEXT,
			'password'   => false,
			'value'      => '',
			'size'       => null,
			'maxlength'  => null,
			'rows'       => 5,
			'columns'    => 72,
			'show-value' => true,
		));
	}

	function build_widget()
	{
		/* Convert value to string if possible */

		$value = $this->_get('value');

		if (is_null($value))
			$value = "";
		elseif (is_int($value) || is_float($value))
			$value = (string) $value;


		if (!is_string($value))
			throw new Exception(sprintf('Text control "%s" can only contain strings or numeric values (%s provided)', $this->get('name'), gettype($value)));


		/* XML tag attributes used both for single line and multiline */

		$attr = array(
			'name'     => $this->get('rendered-name'),
			'id'       => $this->get('id'),
		);
		if ($this->get('extra_attributes')) {
			$attr = array_merge($attr, $this->get('extra_attributes'));
		}

		if ($this->_get('readonly'))
			$attr['readonly'] = 'readonly';

		if ($this->_get('disabled'))
			$attr['disabled'] = 'disabled';


		/* Now differentiate between single and multiline controls */

		if ($this->_get('multiline'))
		{
			/* Output textarea element */

			$rows = $this->_get('rows');
			assert('is_int($rows)');
			$attr['rows'] = (string) $rows;

			$columns = $this->_get('columns');
			assert('is_int($columns)');
			$attr['cols'] = (string) $columns;

			$widget = new AnewtXHTMLTextarea($value, $attr);

			/* Styling */

			if (!$this->_get('required'))
				$widget->add_class('optional');

		} else
		{
			/* Output input element */

			$is_password = $this->_get('password');
			if ($is_password) {
				$attr['type'] = 'password';
			} else {
				$attr['type'] = 'text';
			}

			if ($this->_get('show-value')) {
				$attr['value'] =  $value;
			} else {
				$attr['value'] = '';
			}

			$size = $this->_get('size');
			if (!is_null($size))
			{
				assert('is_int($size);');
				$attr['size'] = (string) $size;
			}

			$maxlength = $this->_get('maxlength');
			if (!is_null($maxlength))
			{
				assert('is_int($maxlength);');
				$attr['maxlength'] = (string) $maxlength;
			}

			$widget = new AnewtXHTMLInput(null, $attr);

			/* Styling */

			$widget->add_class('text');

			if ($is_password)
				$widget->add_class('password');

			if (!$this->_get('required'))
				$widget->add_class('text-optional');
		}

		/* Optional extra class value */
		$class = $this->_get('class');
		if (!is_null($class))
			$widget->add_class($class);

		/* Help text, if any */
		$help = $this->_get('help');
		if (!is_null($help))
		{
			$help_text = to_string($help);
			$widget->set_attribute('title', $help_text);
			$widget->add_class('with-help');
		}

		/* Add secondary label, if any */
		$secondary_label = $this->_get('secondary-label');
		if (is_null($secondary_label))
			$out = $widget;
		else
			$out = ax_fragment($widget, $secondary_label);

		return $out;
	}

	function fill($values) {
		$name = $this->get('name');

		if (array_key_exists($name, $values)) {
			if ($this->get('type') == ANEWT_FORM_CONTROL_TEXT_TYPE_INT) {
				$values[$name] = (int) $values[$name];
			} else if ($this->get('type') == ANEWT_FORM_CONTROL_TEXT_TYPE_FLOAT) {
				$values[$name] = (float) $values[$name];
			}
		}
		return parent::fill($values);
	}

	/** \{
	 * \name Validation methods
	 */

	function is_valid()
	{
		/* If this is a single line text input control, we can do some
		 * preliminary checks to see whether the user has tried to mess things
		 * up by now respecting our value restrictions in the html. This will
		 * not result in pretty error messages, but hey, who's trying to fool
		 * who anyway? */
		if (!$this->_get('multiline') && $this->get('type') == ANEWT_FORM_CONTROL_TEXT_TYPE_TEXT && !$this->is_empty())
		{
			$value = $this->_get('value');

			/* Newlines are invalid */
			if (str_contains($value, "\n") || str_contains($value, "\r")) {
				$this->set('valid', false);
				$this->set('error', 'Invalid newlines');
				return false;
			}

			/* Check maxlength. FIXME: this likeley does not work correctly with
			 * multibyte UTF-8 characters because of strlen() counting bytes
			 * instead of characters... */
			$maxlength = $this->_get('maxlength');
			if (!is_null($maxlength) && strlen($value) > $maxlength) {
				$this->set('valid', false);
				$this->set('error', 'Too long (probably because of multibyte characters)');
				return false;
			}
		}

		/* Now get along with validators and perhaps other validation */
		return parent::is_valid();
	}

	/** \} */
}


/**
 * Text input form control.
 */
class AnewtFormControlText extends AnewtFormControlTextBase
{
	/**
	 * Create a new text form control.
	 *
	 * \param $name
	 *   The name of this control.
	 */
	function __construct($name)
	{
		parent::__construct($name);
		/* Simply inherit AnewtFormControlTextBase */
	}
}


/**
 * Password input form control.
 */
class AnewtFormControlPassword extends AnewtFormControlTextBase
{
	/**
	 * Create a new password form control.
	 *
	 * \param $name
	 *   The name of this control.
	 */
	function __construct($name)
	{
		parent::__construct($name);
		$this->_seed(array(
			'password'   => true,
			'show-value' => false,
		));
	}
}


/**
 * Integer input form control.
 */
class AnewtFormControlInteger extends AnewtFormControlTextBase
{
	/**
	 * Create a new password form control.
	 *
	 * \param $name
	 *   The name of this control.
	 */
	function __construct($name)
	{
		parent::__construct($name);
		$this->_seed(array(
			'type'   => ANEWT_FORM_CONTROL_TEXT_TYPE_INT,
		));
	}
}

/**
 * Floating point input form control.
 */
class AnewtFormControlFloat extends AnewtFormControlTextBase
{
	/**
	 * Create a new password form control.
	 *
	 * \param $name
	 *   The name of this control.
	 */
	function __construct($name)
	{
		parent::__construct($name);
		$this->_seed(array(
			'type'   => ANEWT_FORM_CONTROL_TEXT_TYPE_FLOAT
		));
	}
}


/**
 * Multiline text input form control.
 */
class AnewtFormControlTextMultiline extends AnewtFormControlTextBase
{
	/**
	 * Create a new multiline text form control.
	 *
	 * \param $name
	 *   The name of this control.
	 */
	function __construct($name)
	{
		parent::__construct($name);
		$this->_set('multiline', true);
	}
}


/**
 * Hidden form control.
 *
 * This control only has name, id (optional) and value properties. This control
 * is not visible when rendered.
 */
class AnewtFormControlHidden extends AnewtFormControl
{
	function __construct($name, $value=null) {
		parent::__construct($name);
		$this->set('value', $value);
	}

	function build_widget()
	{
		$value = $this->get('value');
		if (is_array($value)) {
			/* Render arrays by making each element a separate AnewtFormControlHidden
			 * The key must be between brackets in the name, to preserve it.
			 */
			$widget_list = array();
			foreach ($value as $key => $item) {
				$name = sprintf("%s[%s]", $this->get('name'), $key);
				$id = sprintf("%s-%s", $this->get('id'), $key);
				$c = new AnewtFormControlHidden($name);
				$c->_set_form($this->_form);
				$c->set('value', $item);
				$c->set('id', $id);

				$widget_list[] = $c->build_widget();
			}
			return ax_fragment($widget_list);
		} else {
			$attr = array(
				'name'  => $this->get('rendered-name'),
				'id'    => $this->get('id'),
				'value' => (string) $this->get('value'),
				'type'  => 'hidden',
			);

			$widget = new AnewtXHTMLInput(null, $attr);
			return $widget;
		}
	}
}

?>
